@@ -107,10 +107,6 @@ local config = {
   ['url_enabled'] = false
 }
 
-local function trim1(s)
-  return (s:gsub("^%s*(.-)%s*$", "%1"))
-end
-
 local function check_regexp(str, regexp_text)
   local re = rspamd_regexp.create_cached(regexp_text, 'i')
   if re:match(str) then return true end
@@ -239,9 +235,7 @@ local function hfilter(task)
             if total_url_len + 7 > total_parts_len then
               task:insert_result('HFILTER_URL_ONLY', 1.00)
             elseif text_parts_count == 1 and selected_text_part and selected_text_part:get_length() < 1024 then
-              -- We got a single text part with the total length < 1024 symbols.
-              local part_text = selected_text_part:get_content()
-              if part_text and not string.find(trim1(part_text), "\n") then
+              if selected_text_part:get_lines_count() < 2 then
                 task:insert_result('HFILTER_URL_ONELINE', 1.00)
               end
             end
@@ -424,7 +418,6 @@ local symbols_rcpt = {
 local symbols_mid = {
   "HFILTER_MID_NORESOLVE_MX",
   "HFILTER_MID_NORES_A_OR_MX",
-  "HFILTER_MID_NOT_FQDN",
   "HFILTER_MID_NOT_FQDN"
 }
 local symbols_url = {
