@@ -26,6 +26,7 @@
 #include "main.h"
 #include "util.h"
 #include "cfg_file.h"
+#include "cfg_rcl.h"
 #include "message.h"
 #include "utlist.h"
 
@@ -96,37 +97,58 @@
 #define HOSTNAME_HEADER "Hostname"
 #define DELIVER_TO_HEADER "Deliver-To"
 #define NO_LOG_HEADER "Log"
+#define MLEN_HEADER "Message-Length"
 
 static GList *custom_commands = NULL;
 
+ static gchar *
+ rspamd_protocol_header_dup (struct rspamd_task *task, GString *h)
+ {
+ 	gchar *res;
+
+ 	g_assert (h != NULL);
+ 	g_assert (h->len > 0);
+
+ 	res = rspamd_mempool_alloc (task->task_pool, h->len + 1);
+ 	rspamd_strlcpy (res, h->str, h->len + 1);
+
+ 	return res;
+ }
 
 /*
  * Remove <> from the fixed string and copy it to the pool
  */
 static gchar *
-rspamd_protocol_escape_braces (GString *in)
+rspamd_protocol_escape_braces (struct rspamd_task *task, GString *in)
 {
 	gint len = 0;
-	gchar *orig, *p;
+	gchar *orig, *p, *res;
+
+	g_assert (in != NULL);
+	g_assert (in->len > 0);
 
 	orig = in->str;
+
 	while ((g_ascii_isspace (*orig) || *orig ==
 		'<') && orig - in->str < (gint)in->len) {
 		orig++;
 	}
 
-	g_string_erase (in, 0, orig - in->str);
+	res = rspamd_mempool_alloc (task->task_pool,
+			in->len - (orig - in->str));
+
+	p = in->str + in->len - 1;
+	len = in->len - (orig - in->str);
 
-	p = in->str;
 	while ((!g_ascii_isspace (*p) && *p !=
-		'>') && p - in->str < (gint)in->len) {
-		p++;
-		len++;
+		'>') && p > orig) {
+		p--;
+		len--;
 	}
 
-	g_string_truncate (in, len);
+	rspamd_strlcpy (res, orig, len + 1);
 
-	return in->str;
+	return res;
 }
 
 static gboolean
@@ -231,138 +253,125 @@ gboolean
 rspamd_protocol_handle_headers (struct rspamd_task *task,
 	struct rspamd_http_message *msg)
 {
-	gchar *headern, *tmp;
-	gboolean res = TRUE, validh;
+	gchar *headern;
+	GString *hn, *hv;
+	gboolean res = TRUE, validh, fl, has_ip = FALSE;
+	gsize hlen;
 	struct rspamd_http_header *h;
-	InternetAddressList *tmp_addr;
 
 	LL_FOREACH (msg->headers, h)
 	{
-		headern = h->name->str;
+		hn = g_string_new_len (h->name->str, h->name->len);
+		hv = g_string_new_len (h->value->str, h->value->len);
+		headern = hn->str;
+		hlen = hn->len;
 		validh = TRUE;
 
+		g_hash_table_insert (task->request_headers, hn, hv);
+
 		switch (headern[0]) {
 		case 'd':
 		case 'D':
-			if (g_ascii_strcasecmp (headern, DELIVER_TO_HEADER) == 0) {
-				task->deliver_to = rspamd_protocol_escape_braces (h->value);
+			if (g_ascii_strncasecmp (headern, DELIVER_TO_HEADER, hlen) == 0) {
+				task->deliver_to = rspamd_protocol_escape_braces (task, hv);
 				debug_task ("read deliver-to header, value: %s",
 					task->deliver_to);
 			}
 			else {
-				debug_task ("wrong header: %s", headern);
+				debug_task ("wrong header: %v", hn);
 				validh = FALSE;
 			}
 			break;
 		case 'h':
 		case 'H':
-			if (g_ascii_strcasecmp (headern, HELO_HEADER) == 0) {
-				task->helo = h->value->str;
+			if (g_ascii_strncasecmp (headern, HELO_HEADER, hlen) == 0) {
+				task->helo = hv->str;
 				debug_task ("read helo header, value: %s", task->helo);
 			}
-			else if (g_ascii_strcasecmp (headern, HOSTNAME_HEADER) == 0) {
-				task->hostname = h->value->str;
+			else if (g_ascii_strncasecmp (headern, HOSTNAME_HEADER, hlen) == 0) {
+				task->hostname = hv->str;
 				debug_task ("read hostname header, value: %s", task->hostname);
 			}
 			else {
-				debug_task ("wrong header: %s", headern);
+				debug_task ("wrong header: %v", hn);
 				validh = FALSE;
 			}
 			break;
 		case 'f':
 		case 'F':
-			if (g_ascii_strcasecmp (headern, FROM_HEADER) == 0) {
-				task->from_envelope = internet_address_list_parse_string (
-						h->value->str);
-				if (task->from_envelope) {
-#ifdef GMIME24
-					rspamd_mempool_add_destructor (task->task_pool,
-							(rspamd_mempool_destruct_t) g_object_unref,
-							task->from_envelope);
-#else
-					rspamd_mempool_add_destructor (task->task_pool,
-							(rspamd_mempool_destruct_t) internet_address_list_destroy,
-							task->from_envelope);
-#endif
+			if (g_ascii_strncasecmp (headern, FROM_HEADER, hlen) == 0) {
+				if (!rspamd_task_add_sender (task, hv->str)) {
+					msg_err ("bad from header: '%v'", hv);
+					validh = FALSE;
 				}
-				debug_task ("read from header, value: %v", h->value);
 			}
 			else {
-				debug_task ("wrong header: %s", headern);
+				debug_task ("wrong header: %v", hn);
 				validh = FALSE;
 			}
 			break;
 		case 'j':
 		case 'J':
-			if (g_ascii_strcasecmp (headern, JSON_HEADER) == 0) {
-				task->is_json = rspamd_config_parse_flag (h->value->str);
+			if (g_ascii_strncasecmp (headern, JSON_HEADER, hlen) == 0) {
+				fl = rspamd_config_parse_flag (hv->str, hv->len);
+				if (fl) {
+					task->flags |= RSPAMD_TASK_FLAG_JSON;
+				}
+				else {
+					task->flags &= ~RSPAMD_TASK_FLAG_JSON;
+				}
 			}
 			else {
-				debug_task ("wrong header: %s", headern);
+				debug_task ("wrong header: %v", hn);
 				validh = FALSE;
 			}
 			break;
 		case 'q':
 		case 'Q':
-			if (g_ascii_strcasecmp (headern, QUEUE_ID_HEADER) == 0) {
-				task->queue_id = h->value->str;
+			if (g_ascii_strncasecmp (headern, QUEUE_ID_HEADER, hlen) == 0) {
+				task->queue_id = hv->str;
 				debug_task ("read queue_id header, value: %s", task->queue_id);
 			}
 			else {
-				debug_task ("wrong header: %s", headern);
+				debug_task ("wrong header: %v", hn);
 				validh = FALSE;
 			}
 			break;
 		case 'r':
 		case 'R':
-			if (g_ascii_strcasecmp (headern, RCPT_HEADER) == 0) {
-				if (task->rcpt_envelope == NULL) {
-					task->rcpt_envelope = internet_address_list_new ();
-#ifdef GMIME24
-					rspamd_mempool_add_destructor (task->task_pool,
-							(rspamd_mempool_destruct_t) g_object_unref,
-							task->rcpt_envelope);
-#else
-					rspamd_mempool_add_destructor (task->task_pool,
-							(rspamd_mempool_destruct_t) internet_address_list_destroy,
-							task->rcpt_envelope);
-#endif
+			if (g_ascii_strncasecmp (headern, RCPT_HEADER, hlen) == 0) {
+				if (!rspamd_task_add_recipient (task, hv->str)) {
+					msg_err ("bad from header: '%v'", h->value);
+					validh = FALSE;
 				}
-				tmp_addr = internet_address_list_parse_string (h->value->str);
-				internet_address_list_append (task->rcpt_envelope, tmp_addr);
-#ifdef GMIME24
-				g_object_unref (tmp_addr);
-#else
-				internet_address_list_destroy (tmp_addr);
-#endif
-				debug_task ("read rcpt header, value: %v", h->value);
+				debug_task ("read rcpt header, value: %v", hv);
 			}
 			else {
-				debug_task ("wrong header: %s", headern);
+				debug_task ("wrong header: %v", hn);
 				validh = FALSE;
 			}
 			break;
 		case 'i':
 		case 'I':
-			if (g_ascii_strcasecmp (headern, IP_ADDR_HEADER) == 0) {
-				tmp = h->value->str;
-				if (!rspamd_parse_inet_address (&task->from_addr, tmp)) {
-					msg_err ("bad ip header: '%s'", tmp);
+			if (g_ascii_strncasecmp (headern, IP_ADDR_HEADER, hlen) == 0) {
+				if (!rspamd_parse_inet_address (&task->from_addr, hv->str)) {
+					msg_err ("bad ip header: '%v'", hv);
 					return FALSE;
 				}
-				debug_task ("read IP header, value: %s", tmp);
+				debug_task ("read IP header, value: %v", hv);
+				has_ip = TRUE;
 			}
 			else {
-				debug_task ("wrong header: %s", headern);
+				debug_task ("wrong header: %v", hn);
 				validh = FALSE;
 			}
 			break;
 		case 'p':
 		case 'P':
-			if (g_ascii_strcasecmp (headern, PASS_HEADER) == 0) {
-				if (h->value->len == sizeof ("all") - 1 &&
-					g_ascii_strcasecmp (h->value->str, "all") == 0) {
-					task->pass_all_filters = TRUE;
+			if (g_ascii_strncasecmp (headern, PASS_HEADER, hlen) == 0) {
+				if (hv->len == sizeof ("all") - 1 &&
+					g_ascii_strncasecmp (hv->str, "all", hv->len) == 0) {
+					task->flags |= RSPAMD_TASK_FLAG_PASS_ALL;
 					debug_task ("pass all filters");
 				}
 			}
@@ -372,8 +381,8 @@ rspamd_protocol_handle_headers (struct rspamd_task *task,
 			break;
 		case 's':
 		case 'S':
-			if (g_ascii_strcasecmp (headern, SUBJECT_HEADER) == 0) {
-				task->subject = h->value->str;
+			if (g_ascii_strncasecmp (headern, SUBJECT_HEADER, hlen) == 0) {
+				task->subject = hv->str;
 			}
 			else {
 				validh = FALSE;
@@ -381,13 +390,20 @@ rspamd_protocol_handle_headers (struct rspamd_task *task,
 			break;
 		case 'u':
 		case 'U':
-			if (g_ascii_strcasecmp (headern, USER_HEADER) == 0) {
-				task->user = h->value->str;
+			if (g_ascii_strncasecmp (headern, USER_HEADER, hlen) == 0) {
+				/*
+				 * We must ignore User header in case of spamc, as SA has
+				 * different meaning of this header
+				 */
+				if (!RSPAMD_TASK_IS_SPAMC (task)) {
+					task->user = hv->str;
+				}
 			}
-			if (g_ascii_strcasecmp (headern, URLS_HEADER) == 0) {
+			if (g_ascii_strncasecmp (headern, URLS_HEADER, hlen) == 0) {
 				if (h->value->len == sizeof ("extended") - 1 &&
-						g_ascii_strcasecmp (h->value->str, "extended") == 0) {
-					task->extended_urls = TRUE;
+						g_ascii_strncasecmp (hv->str, "extended",
+								hv->len) == 0) {
+					task->flags |= RSPAMD_TASK_FLAG_EXT_URLS;
 					debug_task ("extended urls information");
 				}
 			}
@@ -397,9 +413,26 @@ rspamd_protocol_handle_headers (struct rspamd_task *task,
 			break;
 		case 'l':
 		case 'L':
-			if (g_ascii_strcasecmp (headern, NO_LOG_HEADER) == 0) {
-				if (g_ascii_strcasecmp (h->value->str, "no") == 0) {
-					task->no_log = TRUE;
+			if (g_ascii_strncasecmp (headern, NO_LOG_HEADER, hlen) == 0) {
+				if (g_ascii_strncasecmp (hv->str, "no", hv->len) == 0) {
+					task->flags |= RSPAMD_TASK_FLAG_NO_LOG;
+				}
+			}
+			else {
+				validh = FALSE;
+			}
+			break;
+		case 'm':
+		case 'M':
+			if (g_ascii_strncasecmp (headern, MLEN_HEADER, hlen) == 0) {
+				task->message_len = strtoul (hv->str, NULL, 10);
+
+				if (task->message_len == 0) {
+					msg_err ("Invalid message length header: %v", hv);
+					validh = FALSE;
+				}
+				else {
+					task->flags |= RSPAMD_TASK_FLAG_HAS_CONTROL;
 				}
 			}
 			else {
@@ -407,16 +440,13 @@ rspamd_protocol_handle_headers (struct rspamd_task *task,
 			}
 			break;
 		default:
-			debug_task ("unknown header: %s", headern);
+			debug_task ("unknown header: %v", hn);
 			validh = FALSE;
 			break;
 		}
 
 		if (!validh) {
 			res = FALSE;
-			g_hash_table_replace (task->request_headers,
-				g_string_new_len(h->name->str, h->name->len),
-				g_string_new_len(h->value->str, h->value->len));
 		}
 	}
 
@@ -428,6 +458,120 @@ rspamd_protocol_handle_headers (struct rspamd_task *task,
 		return FALSE;
 	}
 
+	if (task->hostname == NULL || task->hostname[0] == '\0') {
+		/* We assume that hostname is either "unknown" or existing */
+		task->hostname = rspamd_mempool_strdup (task->task_pool, "unknown");
+	}
+
+	if (!has_ip) {
+		task->flags |= RSPAMD_TASK_FLAG_NO_IP;
+	}
+
+	return TRUE;
+}
+
+#define BOOL_TO_FLAG(val, flags, flag) do {									\
+	if ((val)) (flags) |= (flag);											\
+	else (flags) &= ~(flag);												\
+} while(0)
+
+gboolean
+rspamd_protocol_parse_task_flags (rspamd_mempool_t *pool,
+	const ucl_object_t *obj,
+	gpointer ud,
+	struct rspamd_rcl_section *section,
+	GError **err)
+{
+	struct rspamd_rcl_struct_parser *pd = ud;
+	gint *target;
+	const gchar *key;
+	gboolean value;
+
+	target = (gint *)(((gchar *)pd->user_struct) + pd->offset);
+	key = ucl_object_key (obj);
+	value = ucl_object_toboolean (obj);
+
+	if (key != NULL) {
+		if (g_ascii_strcasecmp (key, "pass_all") == 0) {
+			BOOL_TO_FLAG (value, *target, RSPAMD_TASK_FLAG_PASS_ALL);
+		}
+		else if (g_ascii_strcasecmp (key, "no_log") == 0) {
+			BOOL_TO_FLAG (value, *target, RSPAMD_TASK_FLAG_NO_LOG);
+		}
+	}
+
+	return TRUE;
+}
+
+static struct rspamd_rcl_section *control_parser = NULL;
+
+static void
+rspamd_protocol_control_parser_init (void)
+{
+	struct rspamd_rcl_section *sub;
+
+	if (control_parser == NULL) {
+		sub = rspamd_rcl_add_section (&control_parser,
+				"*",
+				NULL,
+				UCL_OBJECT,
+				FALSE,
+				TRUE);
+		/* Default handlers */
+		rspamd_rcl_add_default_handler (sub,
+				"ip",
+				rspamd_rcl_parse_struct_addr,
+				G_STRUCT_OFFSET (struct rspamd_task, from_addr),
+				0);
+		rspamd_rcl_add_default_handler (sub,
+				"from",
+				rspamd_rcl_parse_struct_mime_addr,
+				G_STRUCT_OFFSET (struct rspamd_task, from_envelope),
+				0);
+		rspamd_rcl_add_default_handler (sub,
+				"rcpt",
+				rspamd_rcl_parse_struct_mime_addr,
+				G_STRUCT_OFFSET (struct rspamd_task, rcpt_envelope),
+				0);
+		rspamd_rcl_add_default_handler (sub,
+				"helo",
+				rspamd_rcl_parse_struct_string,
+				G_STRUCT_OFFSET (struct rspamd_task, helo),
+				0);
+		rspamd_rcl_add_default_handler (sub,
+				"user",
+				rspamd_rcl_parse_struct_string,
+				G_STRUCT_OFFSET (struct rspamd_task, user),
+				0);
+		rspamd_rcl_add_default_handler (sub,
+				"pass_all",
+				rspamd_protocol_parse_task_flags,
+				G_STRUCT_OFFSET (struct rspamd_task, flags),
+				0);
+		rspamd_rcl_add_default_handler (sub,
+				"json",
+				rspamd_protocol_parse_task_flags,
+				G_STRUCT_OFFSET (struct rspamd_task, flags),
+				0);
+	}
+}
+
+gboolean
+rspamd_protocol_handle_control (struct rspamd_task *task,
+		const ucl_object_t *control)
+{
+	GError *err = NULL;
+
+	rspamd_protocol_control_parser_init ();
+
+	if (!rspamd_rcl_parse (control_parser, task, task->task_pool,
+			control, &err)) {
+		msg_warn ("cannot parse control block: %e", err);
+		g_error_free (err);
+
+		return FALSE;
+	}
+
 	return TRUE;
 }
 
@@ -439,19 +583,20 @@ rspamd_protocol_handle_request (struct rspamd_task *task,
 
 	if (msg->method == HTTP_SYMBOLS) {
 		task->cmd = CMD_SYMBOLS;
-		task->is_json = FALSE;
+		task->flags &= ~RSPAMD_TASK_FLAG_JSON;
 	}
 	else if (msg->method == HTTP_CHECK) {
 		task->cmd = CMD_CHECK;
-		task->is_json = FALSE;
+		task->flags &= ~RSPAMD_TASK_FLAG_JSON;
 	}
 	else {
-		task->is_json = TRUE;
+		task->flags |= RSPAMD_TASK_FLAG_JSON;
 		ret = rspamd_protocol_handle_url (task, msg);
 	}
 
 	if (msg->flags & RSPAMD_HTTP_FLAG_SPAMC) {
-		task->is_spamc = TRUE;
+		task->flags &= ~RSPAMD_TASK_FLAG_JSON;
+		task->flags |= RSPAMD_TASK_FLAG_SPAMC;
 	}
 
 	return ret;
@@ -492,14 +637,14 @@ struct tree_cb_data {
 /*
  * Callback for writing urls
  */
-static gboolean
+static void
 urls_protocol_cb (gpointer key, gpointer value, gpointer ud)
 {
 	struct tree_cb_data *cb = ud;
 	struct rspamd_url *url = value;
 	ucl_object_t *obj, *elt;
 
-	if (!cb->task->extended_urls) {
+	if (!(cb->task->flags & RSPAMD_TASK_FLAG_EXT_URLS)) {
 		obj = ucl_object_fromlstring (url->host, url->hostlen);
 	}
 	else {
@@ -528,15 +673,13 @@ urls_protocol_cb (gpointer key, gpointer value, gpointer ud)
 			cb->task->message_id,
 			cb->task->user ?
 			cb->task->user : "unknown",
-			rspamd_inet_address_to_string (&cb->task->from_addr),
+			rspamd_inet_address_to_string (cb->task->from_addr),
 			struri (url));
 	}
-
-	return FALSE;
 }
 
 static ucl_object_t *
-rspamd_urls_tree_ucl (GTree *input, struct rspamd_task *task)
+rspamd_urls_tree_ucl (GHashTable *input, struct rspamd_task *task)
 {
 	struct tree_cb_data cb;
 	ucl_object_t *obj;
@@ -545,12 +688,12 @@ rspamd_urls_tree_ucl (GTree *input, struct rspamd_task *task)
 	cb.top = obj;
 	cb.task = task;
 
-	g_tree_foreach (input, urls_protocol_cb, &cb);
+	g_hash_table_foreach (input, urls_protocol_cb, &cb);
 
 	return obj;
 }
 
-static gboolean
+static void
 emails_protocol_cb (gpointer key, gpointer value, gpointer ud)
 {
 	struct tree_cb_data *cb = ud;
@@ -559,12 +702,10 @@ emails_protocol_cb (gpointer key, gpointer value, gpointer ud)
 
 	obj = ucl_object_fromlstring (url->user, url->userlen + url->hostlen + 1);
 	ucl_array_append (cb->top, obj);
-
-	return FALSE;
 }
 
 static ucl_object_t *
-rspamd_emails_tree_ucl (GTree *input, struct rspamd_task *task)
+rspamd_emails_tree_ucl (GHashTable *input, struct rspamd_task *task)
 {
 	struct tree_cb_data cb;
 	ucl_object_t *obj;
@@ -573,7 +714,7 @@ rspamd_emails_tree_ucl (GTree *input, struct rspamd_task *task)
 	cb.top = obj;
 	cb.task = task;
 
-	g_tree_foreach (input, emails_protocol_cb, &cb);
+	g_hash_table_foreach (input, emails_protocol_cb, &cb);
 
 	return obj;
 }
@@ -642,7 +783,10 @@ rspamd_metric_symbol_ucl (struct rspamd_task *task, struct metric *m,
 	ucl_object_t *obj = NULL;
 	const gchar *description = NULL;
 
-	rspamd_printf_gstring (logbuf, "%s,", sym->name);
+	if (logbuf != NULL) {
+		rspamd_printf_gstring (logbuf, "%s,", sym->name);
+	}
+
 	description = g_hash_table_lookup (m->descriptions, sym->name);
 
 	obj = ucl_object_typed_new (UCL_OBJECT);
@@ -691,7 +835,7 @@ rspamd_metric_result_ucl (struct rspamd_task *task,
 	action = mres->action;
 	is_spam = (action == METRIC_ACTION_REJECT);
 
-	if (task->is_skipped) {
+	if (RSPAMD_TASK_IS_SKIPPED (task)) {
 		action_char = 'S';
 	}
 	else if (is_spam) {
@@ -701,15 +845,17 @@ rspamd_metric_result_ucl (struct rspamd_task *task,
 		action_char = 'F';
 	}
 
-	rspamd_printf_gstring (logbuf, "(%s: %c (%s): [%.2f/%.2f] [",
-		m->name, action_char,
-		rspamd_action_to_str (action),
-		mres->score, required_score);
+	if (logbuf != NULL) {
+		rspamd_printf_gstring (logbuf, "(%s: %c (%s): [%.2f/%.2f] [",
+				m->name, action_char,
+				rspamd_action_to_str (action),
+				mres->score, required_score);
+	}
 
 	obj = ucl_object_typed_new (UCL_OBJECT);
 	ucl_object_insert_key (obj,	  ucl_object_frombool (is_spam),
 		"is_spam", 0, false);
-	ucl_object_insert_key (obj,	  ucl_object_frombool (task->is_skipped),
+	ucl_object_insert_key (obj,	  ucl_object_frombool (RSPAMD_TASK_IS_SKIPPED (task)),
 		"is_skipped", 0, false);
 	ucl_object_insert_key (obj, ucl_object_fromdouble (mres->score),
 		"score", 0, false);
@@ -732,22 +878,20 @@ rspamd_metric_result_ucl (struct rspamd_task *task,
 		ucl_object_insert_key (obj, sobj, h, 0, false);
 	}
 
-	/* Cut the trailing comma if needed */
-	if (logbuf->str[logbuf->len - 1] == ',') {
-		logbuf->len--;
-	}
+	if (logbuf != NULL) {
+		/* Cut the trailing comma if needed */
+		if (logbuf->str[logbuf->len - 1] == ',') {
+			logbuf->len--;
+		}
 
-#ifdef HAVE_CLOCK_GETTIME
-	rspamd_printf_gstring (logbuf, "]), len: %z, time: %s, dns req: %d,",
-		task->msg->len, calculate_check_time (&task->tv, &task->ts,
-		task->cfg->clock_res, &task->scan_milliseconds), task->dns_requests);
-#else
-	rspamd_printf_gstring (logbuf, "]), len: %z, time: %s, dns req: %d,",
-		task->msg->len,
-		calculate_check_time (&task->tv, task->cfg->clock_res,
-		&task->scan_milliseconds),
-		task->dns_requests);
-#endif
+		rspamd_printf_gstring (logbuf, "]), len: %z, time: %s, dns req: %d,",
+				task->msg.len,
+				calculate_check_time (task->time_real,
+						task->time_virtual,
+						task->cfg->clock_res,
+						&task->scan_milliseconds),
+				task->dns_requests);
+	}
 
 	return obj;
 }
@@ -843,43 +987,26 @@ rspamd_ucl_tospamc_output (struct rspamd_task *task,
 	}
 }
 
-void
-rspamd_protocol_http_reply (struct rspamd_http_message *msg,
-	struct rspamd_task *task)
+ucl_object_t *
+rspamd_protocol_write_ucl (struct rspamd_task *task, GString *logbuf)
 {
-	GString *logbuf;
 	struct metric_result *metric_res;
+	ucl_object_t *top = NULL, *obj;
 	GHashTableIter hiter;
 	gpointer h, v;
-	ucl_object_t *top = NULL, *obj;
-	gdouble required_score;
-	gint action;
 
-	/* Output the first line - check status */
-	logbuf = g_string_sized_new (BUFSIZ);
-	rspamd_printf_gstring (logbuf,
-		"id: <%s>, qid: <%s>, ",
-		task->message_id,
-		task->queue_id);
+	if (logbuf != NULL) {
+		rspamd_printf_gstring (logbuf,
+				"id: <%s>, qid: <%s>, ",
+				task->message_id,
+				task->queue_id);
 
-	if (task->user) {
-		rspamd_printf_gstring (logbuf, "user: %s, ", task->user);
-	}
-
-	if (!task->no_log) {
-		rspamd_roll_history_update (task->worker->srv->history, task);
-	}
-
-	/* Write custom headers */
-	g_hash_table_iter_init (&hiter, task->reply_headers);
-	while (g_hash_table_iter_next (&hiter, &h, &v)) {
-		GString *hn = (GString *)h, *hv = (GString *)v;
-
-		rspamd_http_message_add_header (msg, hn->str, hv->str);
+		if (task->user) {
+			rspamd_printf_gstring (logbuf, "user: %s, ", task->user);
+		}
 	}
 
 	g_hash_table_iter_init (&hiter, task->results);
-
 	top = ucl_object_typed_new (UCL_OBJECT);
 	/* Convert results to an ucl object */
 	while (g_hash_table_iter_next (&hiter, &h, &v)) {
@@ -892,31 +1019,66 @@ rspamd_protocol_http_reply (struct rspamd_http_message *msg,
 		ucl_object_insert_key (top, rspamd_str_list_ucl (
 				task->messages), "messages", 0, false);
 	}
-	if (g_tree_nnodes (task->urls) > 0) {
+	if (g_hash_table_size (task->urls) > 0) {
 		ucl_object_insert_key (top, rspamd_urls_tree_ucl (task->urls,
-			task), "urls", 0, false);
+				task), "urls", 0, false);
 	}
-	if (g_tree_nnodes (task->emails) > 0) {
+	if (g_hash_table_size (task->emails) > 0) {
 		ucl_object_insert_key (top, rspamd_emails_tree_ucl (task->emails, task),
-			"emails", 0, false);
+				"emails", 0, false);
 	}
 
 	ucl_object_insert_key (top, ucl_object_fromstring (task->message_id),
-		"message-id", 0, false);
+			"message-id", 0, false);
 
-	write_hashes_to_log (task, logbuf);
-	if (!task->no_log) {
+	if (logbuf != NULL) {
+		write_hashes_to_log (task, logbuf);
+	}
+
+	return top;
+}
+
+void
+rspamd_protocol_http_reply (struct rspamd_http_message *msg,
+	struct rspamd_task *task)
+{
+	GString *logbuf;
+	struct metric_result *metric_res;
+	GHashTableIter hiter;
+	gpointer h, v;
+	ucl_object_t *top = NULL;
+	gdouble required_score;
+	gint action;
+
+	/* Output the first line - check status */
+	logbuf = g_string_sized_new (BUFSIZ);
+
+	/* Write custom headers */
+	g_hash_table_iter_init (&hiter, task->reply_headers);
+	while (g_hash_table_iter_next (&hiter, &h, &v)) {
+		GString *hn = (GString *)h, *hv = (GString *)v;
+
+		rspamd_http_message_add_header (msg, hn->str, hv->str);
+	}
+
+	top = rspamd_protocol_write_ucl (task, logbuf);
+
+	if (!(task->flags & RSPAMD_TASK_FLAG_NO_LOG)) {
+		rspamd_roll_history_update (task->worker->srv->history, task);
+	}
+
+	if (!(task->flags & RSPAMD_TASK_FLAG_NO_LOG)) {
 		msg_info ("%v", logbuf);
 	}
 	g_string_free (logbuf, TRUE);
 
 	msg->body = g_string_sized_new (BUFSIZ);
 
-	if (msg->method < HTTP_SYMBOLS && !task->is_spamc) {
+	if (msg->method < HTTP_SYMBOLS && !RSPAMD_TASK_IS_SPAMC (task)) {
 		rspamd_ucl_emit_gstring (top, UCL_EMIT_JSON_COMPACT, msg->body);
 	}
 	else {
-		if (task->is_spamc) {
+		if (RSPAMD_TASK_IS_SPAMC (task)) {
 			rspamd_ucl_tospamc_output (task, top, msg->body);
 		}
 		else {
@@ -946,11 +1108,16 @@ rspamd_protocol_write_reply (struct rspamd_task *task)
 	const gchar *ctype = "application/json";
 
 	msg = rspamd_http_new_message (HTTP_RESPONSE);
-	if (!task->is_json) {
+
+	if (rspamd_http_connection_is_encrypted (task->http_conn)) {
+		msg_info ("<%s> writing encrypted reply", task->message_id);
+	}
+
+	if (!RSPAMD_TASK_IS_JSON (task)) {
 		/* Turn compatibility on */
 		msg->method = HTTP_SYMBOLS;
 	}
-	if (task->is_spamc) {
+	if (RSPAMD_TASK_IS_SPAMC (task)) {
 		msg->flags |= RSPAMD_HTTP_FLAG_SPAMC;
 	}
 
