@@ -118,12 +118,12 @@ gboolean rspamd_upstreams_from_ucl (struct upstream_list *ups,
 rspamd_inet_addr_t* rspamd_upstream_addr (struct upstream *up);
 
 /**
- * Add custom address for an upstream
+ * Add custom address for an upstream (ownership of addr is transferred to upstream)
  * @param up
  * @return
  */
 gboolean rspamd_upstream_add_addr (struct upstream *up,
-		const rspamd_inet_addr_t *addr);
+		rspamd_inet_addr_t *addr);
 
 /**
  * Returns the symbolic name of the upstream
