@@ -25,7 +25,30 @@
 #include "dns.h"
 #include "utlist.h"
 
-/* Public prototypes */
+
+/***
+ * @module rspamd_resolver
+ * This module allows to resolve DNS names from LUA code. All resolving is executed
+ * asynchronously. Here is an example of name resolution:
+ * @example
+local function symbol_callback(task)
+	local host = 'example.com'
+
+	local function dns_cb(resolver, to_resolve, results, err)
+		if not results then
+			rspamd_logger.infox('DNS resolving of %1 failed: %2', host, err)
+			return
+		end
+		for _,r in ipairs(results) do
+			-- r is of type rspamd{ip} here, but it can be converted to string
+			rspamd_logger.infox('Resolved %1 to %2', host, tostring(r))
+		end
+	end
+
+	task:get_resolver():resolve_a(task:get_session(), task:get_mempool(),
+		host, dns_cb)
+end
+ */
 struct rspamd_dns_resolver * lua_check_dns_resolver (lua_State * L);
 void luaopen_dns_resolver (lua_State * L);
 
@@ -91,7 +114,7 @@ lua_dns_callback (struct rdns_reply *reply, gpointer arg)
 	gint i = 0;
 	struct rspamd_dns_resolver **presolver;
 	struct rdns_reply_entry *elt;
-	rspamd_inet_addr_t addr;
+	rspamd_inet_addr_t *addr;
 
 	lua_rawgeti (cd->L, LUA_REGISTRYINDEX, cd->cbref);
 	presolver = lua_newuserdata (cd->L, sizeof (gpointer));
@@ -109,19 +132,15 @@ lua_dns_callback (struct rdns_reply *reply, gpointer arg)
 		{
 			switch (elt->type) {
 			case RDNS_REQUEST_A:
-				addr.af = AF_INET;
-				addr.slen = sizeof (addr.addr.s4);
-				memcpy (&addr.addr.s4.sin_addr, &elt->content.a.addr,
-					sizeof (addr.addr.s4.sin_addr));
-				rspamd_lua_ip_push (cd->L, &addr);
+				addr = rspamd_inet_address_new (AF_INET, &elt->content.a.addr);
+				rspamd_lua_ip_push (cd->L, addr);
+				rspamd_inet_address_destroy (addr);
 				lua_rawseti (cd->L, -2, ++i);
 				break;
 			case RDNS_REQUEST_AAAA:
-				addr.af = AF_INET6;
-				addr.slen = sizeof (addr.addr.s6);
-				memcpy (&addr.addr.s6.sin6_addr, &elt->content.aaa.addr,
-					sizeof (addr.addr.s6.sin6_addr));
-				rspamd_lua_ip_push (cd->L, &addr);
+				addr = rspamd_inet_address_new (AF_INET6, &elt->content.aaa.addr);
+				rspamd_lua_ip_push (cd->L, addr);
+				rspamd_inet_address_destroy (addr);
 				lua_rawseti (cd->L, -2, ++i);
 				break;
 			case RDNS_REQUEST_PTR:
@@ -167,6 +186,12 @@ lua_dns_callback (struct rdns_reply *reply, gpointer arg)
 	luaL_unref (cd->L, LUA_REGISTRYINDEX, cd->cbref);
 }
 
+/***
+ * @function rspamd_resolver.init(ev_base, config)
+ * @param {event_base} ev_base event base used for asynchronous events
+ * @param {rspamd_config} config rspamd configuration parameters
+ * @return {rspamd_resolver} new resolver object associated with the specified base
+ */
 static int
 lua_dns_resolver_init (lua_State *L)
 {
@@ -267,6 +292,15 @@ lua_dns_resolver_resolve_common (lua_State *L,
 
 }
 
+/***
+ * @method resolver:resolve_a(session, pool, host, callback)
+ * Resolve A record for a specified host.
+ * @param {async_session} session asynchronous session normally associated with rspamd task (`task:get_session()`)
+ * @param {mempool} pool memory pool for storing intermediate data
+ * @param {string} host name to resolve
+ * @param {function} callback callback function to be called upon name resolution is finished; must be of type `function (resolver, to_resolve, results, err)`
+ * @return {boolean} `true` if DNS request has been scheduled
+ */
 static int
 lua_dns_resolver_resolve_a (lua_State *L)
 {
@@ -285,6 +319,15 @@ lua_dns_resolver_resolve_a (lua_State *L)
 	return 1;
 }
 
+/***
+ * @method resolver:resolve_ptr(session, pool, ip, callback)
+ * Resolve PTR record for a specified host.
+ * @param {async_session} session asynchronous session normally associated with rspamd task (`task:get_session()`)
+ * @param {mempool} pool memory pool for storing intermediate data
+ * @param {string} ip name to resolve in string form (e.g. '8.8.8.8' or '2001:dead::')
+ * @param {function} callback callback function to be called upon name resolution is finished; must be of type `function (resolver, to_resolve, results, err)`
+ * @return {boolean} `true` if DNS request has been scheduled
+ */
 static int
 lua_dns_resolver_resolve_ptr (lua_State *L)
 {
@@ -303,6 +346,15 @@ lua_dns_resolver_resolve_ptr (lua_State *L)
 	return 1;
 }
 
+/***
+ * @method resolver:resolve_txt(session, pool, host, callback)
+ * Resolve TXT record for a specified host.
+ * @param {async_session} session asynchronous session normally associated with rspamd task (`task:get_session()`)
+ * @param {mempool} pool memory pool for storing intermediate data
+ * @param {string} host name to get TXT record for
+ * @param {function} callback callback function to be called upon name resolution is finished; must be of type `function (resolver, to_resolve, results, err)`
+ * @return {boolean} `true` if DNS request has been scheduled
+ */
 static int
 lua_dns_resolver_resolve_txt (lua_State *L)
 {
@@ -321,6 +373,15 @@ lua_dns_resolver_resolve_txt (lua_State *L)
 	return 1;
 }
 
+/***
+ * @method resolver:resolve_mx(session, pool, host, callback)
+ * Resolve MX record for a specified host.
+ * @param {async_session} session asynchronous session normally associated with rspamd task (`task:get_session()`)
+ * @param {mempool} pool memory pool for storing intermediate data
+ * @param {string} host name to get MX record for
+ * @param {function} callback callback function to be called upon name resolution is finished; must be of type `function (resolver, to_resolve, results, err)`
+ * @return {boolean} `true` if DNS request has been scheduled
+ */
 static int
 lua_dns_resolver_resolve_mx (lua_State *L)
 {
@@ -339,6 +400,7 @@ lua_dns_resolver_resolve_mx (lua_State *L)
 	return 1;
 }
 
+/* XXX: broken currently */
 static int
 lua_dns_resolver_resolve (lua_State *L)
 {
