@@ -11,9 +11,6 @@ void rspamd_url_test_func (void);
 /* Memory pools */
 void rspamd_mem_pool_test_func (void);
 
-/* Expressions */
-void rspamd_expression_test_func (void);
-
 /* Fuzzy hashes */
 void rspamd_fuzzy_test_func (void);
 
@@ -39,4 +36,10 @@ void rspamd_upstream_test_func (void);
 
 void rspamd_shingles_test_func (void);
 
+void rspamd_http_test_func (void);
+
+void rspamd_lua_test_func (void);
+
+void rspamd_cryptobox_test_func (void);
+
 #endif
